@@ -20,16 +20,16 @@
 # Helper functions for the ROC curves. These functions should not be called directly as they peform very specific tasks and do nearly no argument validity checks. Not documented in RD and not exported.
 
 # returns a list of sensitivities (se) and specificities (sp) for the given data. Robust algorithm
-roc.utils.perfs.all.safe <- function(thresholds, controls, cases, direction) {
-  perf.matrix <- sapply(thresholds, roc.utils.perfs, controls=controls, cases=cases, direction=direction)
-  #stopifnot(identical(roc.utils.perfs.all.fast(thresholds, controls, cases, direction), list(se=perf.matrix[2,], sp=perf.matrix[1,])))
+roc_utils_perfs_all_safe <- function(thresholds, controls, cases, direction) {
+  perf.matrix <- sapply(thresholds, roc_utils_perfs, controls=controls, cases=cases, direction=direction)
+  #stopifnot(identical(roc_utils_perfs_all_fast(thresholds, controls, cases, direction), list(se=perf.matrix[2,], sp=perf.matrix[1,])))
   return(list(se=perf.matrix[2,], sp=perf.matrix[1,]))
 }
 
 
-roc.utils.perfs.all.test <- function(thresholds, controls, cases, direction) {
-	perfs.safe <- roc.utils.perfs.all.safe(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
-	perfs.fast <- roc.utils.perfs.all.fast(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
+roc_utils_perfs_all_test <- function(thresholds, controls, cases, direction) {
+	perfs.safe <- roc_utils_perfs_all_safe(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
+	perfs.fast <- roc_utils_perfs_all_fast(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
 	perfs.C <- rocUtilsPerfsAllC(thresholds=thresholds, controls=controls, cases=cases, direction=direction)
 	if (! (identical(perfs.safe, perfs.fast) && identical(perfs.safe, perfs.C))) {
 		sessionInfo <- sessionInfo()
@@ -41,7 +41,7 @@ roc.utils.perfs.all.test <- function(thresholds, controls, cases, direction) {
 
 
 # returns a list of sensitivities (se) and specificities (sp) for the given data. Fast algorithm
-roc.utils.perfs.all.fast <- function(thresholds, controls, cases, direction) {
+roc_utils_perfs_all_fast <- function(thresholds, controls, cases, direction) {
   ncontrols <- length(controls)
   ncases <- length(cases)
   predictor <- c(controls, cases)
@@ -76,16 +76,16 @@ roc.utils.perfs.all.fast <- function(thresholds, controls, cases, direction) {
   return(list(se=se, sp=sp))
 }
 
-# As roc.utils.perfs.all but returns an "old-style" matrix (pre-fast-algo-compatible)
-#roc.utils.perfs.all.matrix <- function(...) {
-#  perfs <- roc.utils.perfs.all(...)
+# As roc_utils_perfs.all but returns an "old-style" matrix (pre-fast-algo-compatible)
+#roc_utils_perfs.all.matrix <- function(...) {
+#  perfs <- roc_utils_perfs.all(...)
 #  return(matrix(c(perfs$sp, perfs$se), nrow=2, byrow=TRUE))
 #}
 
 # returns a vector with two elements, sensitivity and specificity, given the threshold at which to evaluate the performance, the values of controls and cases and the direction of the comparison, a character '>' or '<' as controls CMP cases
-# sp <- roc.utils.perfs(...)[1,]
-# se <- roc.utils.perfs(...)[2,]
-roc.utils.perfs <- function(threshold, controls, cases, direction) {
+# sp <- roc_utils_perfs(...)[1,]
+# se <- roc_utils_perfs(...)[2,]
+roc_utils_perfs <- function(threshold, controls, cases, direction) {
   if (direction == '>') {
     tp <- sum(cases <= threshold)
     tn <- sum(controls > threshold)
@@ -98,8 +98,8 @@ roc.utils.perfs <- function(threshold, controls, cases, direction) {
   return(c(sp=tn/length(controls), se=tp/length(cases)))
 }
 
-# as roc.utils.perfs, but for densities
-roc.utils.perfs.dens <- function(threshold, x, dens.controls, dens.cases, direction) {
+# as roc_utils_perfs, but for densities
+roc_utils_perfs_dens <- function(threshold, x, dens.controls, dens.cases, direction) {
   if (direction == '>') {
     tp <- sum(dens.cases[x <= threshold])
     tn <- sum(dens.controls[x > threshold])
@@ -113,7 +113,7 @@ roc.utils.perfs.dens <- function(threshold, x, dens.controls, dens.cases, direct
 }
 
 # return the thresholds to evaluate in the ROC curve, given the 'predictor' values. Returns all unique values of 'predictor' plus 2 extreme values
-roc.utils.thresholds <- function(predictor, direction) {
+roc_utils_thresholds <- function(predictor, direction) {
   unique.candidates <- sort(unique(predictor))
   thresholds1 <- (c(-Inf, unique.candidates) + c(unique.candidates, +Inf))/2
   thresholds2 <- (c(-Inf, unique.candidates)/2 + c(unique.candidates, +Inf)/2)
@@ -172,7 +172,7 @@ roc.utils.thresholds <- function(predictor, direction) {
 }
 
 # Find all the local maximas of the ROC curve. Returns a logical vector
-roc.utils.max.thresholds.idx <- function(thresholds, sp, se) {
+roc_utils_max_thresholds_idx <- function(thresholds, sp, se) {
   reversed <- FALSE
   if (is.unsorted(sp)) {
     # make sure SP is sorted increasingly, and sort thresholds accordingly
@@ -183,7 +183,7 @@ roc.utils.max.thresholds.idx <- function(thresholds, sp, se) {
   }
   # TODO: find whether the duplicate check is still needed.
   # Should have been fixed by passing only c(controls, cases)
-  # instead of whole 'predictor' to roc.utils.thresholds in roc.default
+  # instead of whole 'predictor' to roc_utils_thresholds in roc.default
   # but are there other potential issues like that?
   dup <- duplicated(data.frame(sp, se))
   thresholds <- thresholds[!dup]
@@ -225,8 +225,38 @@ roc.utils.max.thresholds.idx <- function(thresholds, sp, se) {
   return(local.maximas)
 }
 
+detect.env.true <- function(x) {
+	value <- Sys.getenv(x)
+	if (value == "T" || value == "True" || value == "TRUE" || value == "true") {
+		return(TRUE)
+	}
+	else {
+		return(FALSE)
+	}
+}
+
+# Detect if _R_CHECK_LENGTH_1_CONDITION_  or _R_CHECK_LENGTH_1_LOGIC2_  are set
+# to "True" values, which would break some progress bars.
+# See https://cran.r-project.org/doc/manuals/r-devel/R-ints.html and #97
+# Return True or False accordingly
+roc_utils_dumb_progress_bar <- function() {
+	if (detect.env.true("_R_CHECK_LENGTH_1_CONDITION_") || detect.env.true("_R_CHECK_LENGTH_1_LOGIC2_")) {
+		return(TRUE)
+	}
+	else {
+		return(FALSE)
+	}
+}
+
 # Define which progress bar to use
-roc.utils.get.progress.bar <- function(name = getOption("pROCProgress")$name, title = "Bootstrap", label = "", width = getOption("pROCProgress")$width, char = getOption("pROCProgress")$char, style = getOption("pROCProgress")$style, ...) {
+roc_utils_get_progress_bar <- function(name = getOption("pROCProgress")$name, title = "Bootstrap", label = "", width = getOption("pROCProgress")$width, char = getOption("pROCProgress")$char, style = getOption("pROCProgress")$style, ...) {
+	
+  if (roc_utils_dumb_progress_bar()) {
+    # If the length 1 checks are on, we need to return only
+    # the progress bar name
+    return(getOption("pROCProgress")$name)
+  }
+  # Otherwise proceed normally
   if (name == "tk") { # load tcltk if possible
     if (!requireNamespace("tcltk")) {
       # If tcltk cannot be loaded fall back to default text progress bar
@@ -255,7 +285,7 @@ roc.utils.get.progress.bar <- function(name = getOption("pROCProgress")$name, ti
 }
 
 # sort roc curve. Make sure specificities are increasing.
-sort.roc <- function(roc) {
+sort_roc <- function(roc) {
   if (is.unsorted(roc$specificities)) {
     roc$sensitivities <- rev(roc$sensitivities)
     roc$specificities <- rev(roc$specificities)
@@ -265,7 +295,7 @@ sort.roc <- function(roc) {
 }
 
 # sort smoothed roc curve. Make sure specificities are increasing.
-sort.smooth.roc <- function(roc) {
+sort_smooth_roc <- function(roc) {
   if (is.unsorted(roc$specificities)) {
     roc$sensitivities <- rev(roc$sensitivities)
     roc$specificities <- rev(roc$specificities)
@@ -273,47 +303,67 @@ sort.smooth.roc <- function(roc) {
   return(roc)
 }
 
+# The list of valid coordinate arguments, without 'thresholds'
+roc.utils.valid.coords <- c("specificity", "sensitivity", "accuracy",
+	"tn", "tp", "fn", "fp",
+	"npv", "ppv", "fdr",
+	"fpr", "tpr", "tnr", "fnr", 
+	"1-specificity", "1-sensitivity", "1-accuracy", "1-npv", "1-ppv",
+	"precision", "recall",
+	"youden", "closest.topleft")
+
 # Arguments which can be returned by coords
 # @param threshold: FALSE for smooth.roc where threshold isn't valid
-roc.utils.match.coords.ret.args <- function(x, threshold = TRUE) {
-  valid.ret.args <- c("specificity", "sensitivity", "accuracy",
-  					"tn", "tp", "fn", "fp",
-  					"npv", "ppv", "fdr",
-  					"fpr", "tpr", "tnr", "fnr", 
-  					"1-specificity", "1-sensitivity", "1-accuracy", "1-npv", "1-ppv",
-  					"precision", "recall",
-  					"youden", "closest.topleft")
-  if ("all" %in% x) {
-  	if (length(x) > 1) {
-  		stop("ret='all' can't be used with other 'ret' options.")
-  	}
-  	x <- valid.ret.args
-  	if (threshold) {
-  		x <- c("threshold", x)
-  	}
-  }
-  if (threshold) {
-  	valid.ret.args <- c("threshold", valid.ret.args)
-  }
-  x <- replace(x, x=="topleft", "closest.topleft")
-  x <- replace(x, x=="t", "threshold")
-  x <- replace(x, x=="npe", "1-npv")
-  x <- replace(x, x=="ppe", "1-ppv")
-  match.arg(x, valid.ret.args, several.ok=TRUE)
+roc_utils_match_coords_ret_args <- function(x, threshold = TRUE) {
+	valid.ret.args <- roc.utils.valid.coords
+	if (threshold) {
+		valid.ret.args <- c("threshold", valid.ret.args)
+	}
+
+	if ("all" %in% x) {
+		if (length(x) > 1) {
+			stop("ret='all' can't be used with other 'ret' options.")
+		}
+		x <- valid.ret.args
+	}
+	x <- replace(x, x=="topleft", "closest.topleft")
+	x <- replace(x, x=="t", "threshold")
+	x <- replace(x, x=="npe", "1-npv")
+	x <- replace(x, x=="ppe", "1-ppv")
+	return(match.arg(x, valid.ret.args, several.ok=TRUE))
+}
+
+# Arguments which can be used as input for coords
+# @param threshold: FALSE for smooth.roc where threshold isn't valid
+roc_utils_match_coords_input_args <- function(x, threshold = TRUE) {
+	valid.args <- roc.utils.valid.coords
+	if (threshold) {
+		valid.args <- c("threshold", valid.args)
+	}
+	x <- replace(x, x=="topleft", "closest.topleft")
+	x <- replace(x, x=="t", "threshold")
+	x <- replace(x, x=="npe", "1-npv")
+	x <- replace(x, x=="ppe", "1-ppv")
+	matched <- match.arg(x, valid.args, several.ok=FALSE)
+	# We only handle monotone coords
+	if (! coord.is.monotone[matched]) {
+		stop(sprintf("Coordinate '%s' is not monotone and cannot be used as input.", matched))
+	}
+	return(matched)
 }
 
 # Compute the min/max for partial AUC
 # ... with an auc
-roc.utils.min.partial.auc.auc <- function(auc) {
-  roc.utils.min.partial.auc(attr(auc, "partial.auc"), attr(auc, "percent"))
+roc_utils_min_partial_auc_auc <- function(auc) {
+  roc_utils_min_partial_auc(attr(auc, "partial.auc"), attr(auc, "percent"))
 }
 
-roc.utils.max.partial.auc.auc <- function(roc) {
-  roc.utils.max.partial.auc(attr(auc, "partial.auc"), attr(auc, "percent"))
+roc_utils_max_partial_auc_auc <- function(roc) {
+  roc_utils_max_partial_auc(attr(auc, "partial.auc"), attr(auc, "percent"))
 }
 
 # ... with partial.auc/percent
-roc.utils.min.partial.auc <- function(partial.auc, percent) {
+roc_utils_min_partial_auc <- function(partial.auc, percent) {
   if (!identical(partial.auc, FALSE)) {
     min <- sum(ifelse(percent, 100, 1)-partial.auc)*abs(diff(partial.auc))/2/ifelse(percent, 100, 1)
   }
@@ -323,7 +373,7 @@ roc.utils.min.partial.auc <- function(partial.auc, percent) {
   return(min)
 }
 
-roc.utils.max.partial.auc <- function(partial.auc, percent) {
+roc_utils_max_partial_auc <- function(partial.auc, percent) {
   if (!identical(partial.auc, FALSE)) {
     max <- abs(diff(partial.auc))
   }
@@ -336,7 +386,7 @@ roc.utils.max.partial.auc <- function(partial.auc, percent) {
 # Checks if the 
 # Input: roc object
 # Output: boolean, true the curve reaches 100%/100%, false otherwise
-roc.utils.is.perfect.curve <- function(roc) {
+roc_utils_is_perfect_curve <- function(roc) {
 	best.point <- max(roc$sensitivities + roc$specificities) / ifelse(roc$percent, 100, 1)
 	return(abs(best.point - 2) < .Machine$double.eps ^ 0.5) # or best.point == 2, with numerical tolerance
 }
@@ -372,11 +422,15 @@ load.suggested.package <- function(pkg) {
 
 
 # Calculate coordinates
-# @param substr.percent: 1 or 100
-# @param se, sp
-# @param ncases, ncontrols
+# @param roc: the roc curve, used to guess if data is in percent and number of cases and controls.
+# @param thr, se, sp
+# @param best.weights: see coords 
 # @return data.frame
-roc.utils.calc.coords <- function(substr.percent, thr, se, sp, ncases, ncontrols, best.weights) {
+roc_utils_calc_coords <- function(roc, thr, se, sp, best.weights) {
+	ncases <- ifelse(methods::is(roc, "smooth.roc"), length(attr(roc, "roc")$cases), length(roc$cases))
+	ncontrols <- ifelse(methods::is(roc, "smooth.roc"), length(attr(roc, "roc")$controls), length(roc$controls))
+	substr.percent <- ifelse(roc$percent, 100, 1)
+	
 	tp <- se * ncases / substr.percent
 	fn <- ncases - tp
 	tn <- sp * ncontrols / substr.percent
@@ -392,10 +446,10 @@ roc.utils.calc.coords <- function(substr.percent, thr, se, sp, ncases, ncontrols
 	tnr <- sp
 	fnr <- substr.percent * fn / (tp + fn)
 	fdr <- substr.percent * fp / (tp + fp)
-	youden <- roc.utils.optim.crit(se, sp, substr.percent, best.weights, "youden")
-	closest.topleft <- - roc.utils.optim.crit(se, sp, substr.percent, best.weights, "closest.topleft") / substr.percent
+	youden <- roc_utils_optim_crit(se, sp, substr.percent, best.weights, "youden")
+	closest.topleft <- - roc_utils_optim_crit(se, sp, substr.percent, best.weights, "closest.topleft") / substr.percent
 	
-	return(rbind(
+	return(cbind(
 		threshold=thr,
 		sensitivity=se, 
 		specificity=sp, 
@@ -419,7 +473,8 @@ roc.utils.calc.coords <- function(substr.percent, thr, se, sp, ncases, ncontrols
 		precision=precision,
 		recall=recall,
 		youden=youden,
-		closest.topleft=closest.topleft))
+		closest.topleft=closest.topleft
+	))
 }
 
 # Match arbitrary user-supplied thresholds to the threshold of the ROC curve.
@@ -430,7 +485,7 @@ roc.utils.calc.coords <- function(substr.percent, thr, se, sp, ncases, ncontrols
 # @param roc: the roc curve
 # @param x: the threshold to determine indices
 # @return integer vector of indices along roc$thresholds/roc$se/roc$sp.
-roc.utils.thr.idx <- function(roc, x) {
+roc_utils_thr_idx <- function(roc, x) {
 	cut_points <- sort(unique(roc$predictor))
 	thr_idx <- rep(NA_integer_, length(x))
 	if (roc$direction == "<") {
@@ -467,7 +522,7 @@ roc.utils.thr.idx <- function(roc, x) {
 # @param weights: see coords(best.weights=)
 # @param method: youden or closest.topleft coords(best.method=)
 # @return numeric vector along roc$thresholds/roc$se/roc$sp.
-roc.utils.optim.crit <- function(se, sp, max, weights, method) {
+roc_utils_optim_crit <- function(se, sp, max, weights, method) {
 	if (is.numeric(weights) && length(weights) == 2) {
 		r <- (1 - weights[2]) / (weights[1] * weights[2]) # r should be 1 by default
 	}
@@ -485,4 +540,113 @@ roc.utils.optim.crit <- function(se, sp, max, weights, method) {
 		optim.crit <- - ((max - se)^2 + r * (max - sp)^2)
 	}
 	return(optim.crit)
-}
\ No newline at end of file
+}
+
+coord.is.monotone <- c(
+	"threshold"=TRUE,
+	"sensitivity"=TRUE,
+	"specificity"=TRUE,
+	"accuracy"=FALSE,
+	"tn"=TRUE, 
+	"tp"=TRUE,
+	"fn"=TRUE,
+	"fp"=TRUE,       
+	"npv"=FALSE,
+	"ppv"=FALSE,
+	"tpr"=TRUE,
+	"tnr"=TRUE,
+	"fpr"=TRUE,
+	"fnr"=TRUE,
+	"fdr"=FALSE,
+	"1-specificity"=TRUE,
+	"1-sensitivity"=TRUE,
+	"1-accuracy"=FALSE,
+	"1-npv"=FALSE,
+	"1-ppv"=FALSE,
+	"precision"=FALSE,
+	"recall"=TRUE,
+	"youden"=FALSE,
+	"closest.topleft"=FALSE
+)
+
+coord.is.decreasing <- c(
+	"threshold"=NA, # Depends on direction
+	"sensitivity"=TRUE,
+	"specificity"=FALSE,
+	"accuracy"=NA,
+	"tn"=FALSE, 
+	"tp"=TRUE,
+	"fn"=FALSE,
+	"fp"=TRUE,       
+	"npv"=NA,
+	"ppv"=NA,
+	"tpr"=TRUE,
+	"tnr"=FALSE,
+	"fpr"=TRUE,
+	"fnr"=FALSE,
+	"fdr"=NA,
+	"1-specificity"=TRUE,
+	"1-sensitivity"=FALSE,
+	"1-accuracy"=NA,
+	"1-npv"=NA,
+	"1-ppv"=NA,
+	"precision"=NA,
+	"recall"=TRUE,
+	"youden"=NA,
+	"closest.topleft"=NA
+)
+
+# Get response and predictor(s) from a formula.
+# This function takes care of all the logic to handle
+# weights, subset, na.action etc. It handles formulas with 
+# and without data. It rejects weights and certain na.actions.
+# @param formula
+# @param data
+# @param data.missing 
+# @param call the call from the parent function
+# @param ... the ... from the parent function
+# @return a list with 3 elements: response (vector), predictor.names (character),
+#         predictors (data.frame).
+roc_utils_extract_formula <- function(formula, data, data.missing, call, ...) {
+	# Get predictors (easy)
+	if (data.missing) {
+		predictors <- attr(terms(formula), "term.labels")
+	}
+	else {
+		predictors <- attr(terms(formula, data = data), "term.labels")
+	}
+	
+	indx <- match(c("formula", "data", "weights", "subset", "na.action"), names(call), nomatch=0)
+	if (indx[1] == 0) {
+		stop("A formula argument is required")
+	}
+	# Keep the standard arguments and run them in model.frame
+	temp <- call[c(1,indx)]
+	temp[[1]] <- as.name("model.frame")
+	# Only na.pass and na.fail should be used
+	if (indx[5] != 0) {
+		na.action.value = as.character(call[indx[5]])
+		if (! na.action.value %in% c("na.pass", "na.fail")) {
+			warning(paste0(sprintf("Value %s of na.action is not supported ", na.action.value),
+						   "and will break pairing in roc.test and are.paired. ",
+						   "Please use 'na.rm = TRUE' instead."))
+		}
+	}
+	else {
+		temp$na.action = "na.pass"
+	}
+	# Adjust call with data from caller
+	if (data.missing) {
+		temp$data <- NULL
+	}
+	
+	# Run model.frame in the parent
+	m <- eval.parent(temp, n = 2)
+	
+	if (!is.null(model.weights(m))) stop("weights are not supported")
+	
+	return(list(response.name = names(m)[1],
+				response = model.response(m),
+				predictor.names = predictors,
+				predictors = m[predictors]))
+}
